<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0" xmlns:xsl="http://www.w3.org/1999/XSL/Transform" xmlns="">

<xsl:output method="text" encoding="utf-8" omit-xml-declaration="yes" indent="no"/>

	<!-- DIAGRAM -->

	<xsl:template match="sdDiagram">
		<!-- comment -->
		MODULE main<!--<xsl:value-of select="name"/>-->
		<![CDATA[-- State transition diagram representation ]]>
		VAR 
		<![CDATA[-- states, events, actions, conditions ]]>
		<xsl:apply-templates select="components" mode="var"/>		
		<xsl:apply-templates select="innerComponents" mode="var"/>
		
		<xsl:apply-templates select="components" mode="assign" />
		
		TRANS
		<![CDATA[-- transitions ]]>
			case
			<xsl:apply-templates select="components" mode="trans"/>
		<![CDATA[-- for all the not covered case on the diagram ]]>
			1: next(state) = state &amp; next(event) = event <xsl:apply-templates select="//action" mode="nottrans" />;
			esac
		--end MODULE	
		
		LTLSPEC
		
	</xsl:template>
	
	<!-- VAR -->
	
	<xsl:template match="components" mode="var">
		state : {<xsl:for-each select="state"><xsl:if test="position()!=last()"><xsl:value-of select="name" />,</xsl:if><xsl:if test="position()=last()"><xsl:value-of select="name" /></xsl:if></xsl:for-each>};
	</xsl:template>

	<xsl:template match="innerComponents" mode="var">
		event : {<xsl:for-each select="event"><xsl:value-of select="name" />, </xsl:for-each>null};
		<xsl:apply-templates select="action" mode="var"/>
		<xsl:text>&#10;</xsl:text>
		<xsl:apply-templates select="condition" mode="var"/>
	</xsl:template>
	
	<xsl:template match="action" mode="var">
		<xsl:if test="position()='1'">action :{<xsl:value-of select="name" />
		</xsl:if>
		<xsl:if test="position() &gt; '1'">
			<xsl:if test="position() &lt; last()">, <xsl:value-of select="name" /></xsl:if>
		</xsl:if>
		<xsl:if test="position()=last()">
			<xsl:if test="position()='1'">, null};</xsl:if>
			<xsl:if test="position() &gt; '1'">, <xsl:value-of select="name" />, null};</xsl:if>
		</xsl:if>
	</xsl:template>
	
	<xsl:template match="condition" mode="var">
		<xsl:if test="position()='1'">condition :{<xsl:value-of select="name" />
		</xsl:if>
		<xsl:if test="position() &gt; '1'">
			<xsl:if test="position() &lt; last()">, <xsl:value-of select="name" /></xsl:if>
		</xsl:if>
		<xsl:if test="position()=last()">
			<xsl:if test="position()='1'">, null};</xsl:if>
			<xsl:if test="position() &gt; '1'">, <xsl:value-of select="name" />, null};</xsl:if>
		</xsl:if>
	</xsl:template>	
	<!-- TRANS -->
	
	<xsl:template match="components" mode="trans">
		<xsl:apply-templates select="transition" mode="trans"/>
	</xsl:template>
	
	<xsl:template match="transition" mode="trans">
		<!-- se la transitione non comprende un macrostato sia in origine che in destinazione allora la processo normalmente -->
		<xsl:if test="not(contains(origin/@identifierName,'macrostate')) and not(contains(destination/@identifierName,'macrostate'))">
			<xsl:apply-templates select="origin" mode="trans"/> <xsl:apply-templates select="event" mode="trans"/> <xsl:apply-templates select="condition" mode="trans"/> : <xsl:apply-templates select="destination" mode="trans"/> <xsl:apply-templates select="action" mode="trans"/>;
		</xsl:if>
		<!-- se la transizione contiene un macrostato in origine ma non in destinazione allora devo scrivere una transizione 
		per tutti gli stati del macrostato verso lo stato destinazione-->
		<xsl:if test="(contains(origin/@identifierName,'macrostate')) and not(contains(destination/@identifierName,'macrostate'))">
			<xsl:for-each select="origin/states/state">
				state = <xsl:value-of select="name"/> <xsl:apply-templates select="../../../event" mode="trans"/> <xsl:apply-templates select="../../../condition" mode="trans"/> : <xsl:apply-templates select="../../../destination" mode="trans"/> <xsl:apply-templates select="../../../action" mode="trans"/>;
			</xsl:for-each>
		</xsl:if>
		<!-- se la transizione contiene un macrostato in destinazione ma non in origine allora devo scrivere una transizione 
		che parte dallo stato di origine e finisce nello stato iniziale del macrostato di destinazione-->
		<xsl:if test="not(contains(origin/@identifierName,'macrostate')) and (contains(destination/@identifierName,'macrostate'))">
				<xsl:apply-templates select="origin" mode="trans"/> <xsl:apply-templates select="event" mode="trans"/> <xsl:apply-templates select="condition" mode="trans"/> : next(state) = <xsl:value-of select="destination/initState/name"/> <xsl:apply-templates select="action" mode="trans"/>;
		</xsl:if>
		<!-- se la transizione contiene un macrostato in origine e anche uno in destinazione allora devo scrivere una transizione 
		per tutti gli stati del macrostato origine verso lo stato iniziale del macrostato di destinazione-->
		<xsl:if test="(contains(origin/@identifierName,'macrostate')) and (contains(destination/@identifierName,'macrostate'))">
			<xsl:for-each select="origin/states/state">
				state = <xsl:value-of select="name"/> <xsl:apply-templates select="../../../event" mode="trans"/> <xsl:apply-templates select="../../../condition" mode="trans"/> : next(state) = <xsl:value-of select="../../../destination/initState/name"/> <xsl:apply-templates select="../../../action" mode="trans"/>;
			</xsl:for-each>
		</xsl:if>
		
		
	</xsl:template>
	
	
	<xsl:template match="origin" mode="trans"> state = <xsl:value-of select="name"/> </xsl:template>
	
	<xsl:template match="event" mode="trans"> &amp; event = <xsl:value-of select="name"/>  </xsl:template>
	
	<xsl:template match="event" mode="trans"> &amp; event = <xsl:value-of select="name"/>  </xsl:template>
	
	<xsl:template match="condition" mode="trans"> &amp; condition = <xsl:value-of select="name"/> </xsl:template>
	
	<xsl:template match="destination" mode="trans"> next(state) = <xsl:value-of select="name"/> </xsl:template>
	
	<xsl:template match="action" mode="trans"> &amp; next(action) = <xsl:value-of select="name"/> </xsl:template>
	
	<xsl:template match="action" mode="nottrans"><xsl:if test="position()='1'">&amp; next(action) = null</xsl:if></xsl:template> 
	<!-- ASSIGN -->
	
	<xsl:template match="components" mode="assign">
		<xsl:apply-templates select="state" mode="assign" />	
	</xsl:template>
	
	<xsl:template match="state" mode="assign">
		<xsl:if test="contains(isInit,'true')">
			<![CDATA[-- assign initial state ]]>
			ASSIGN
			init(state) := <xsl:value-of select="name" />;
		</xsl:if>
	</xsl:template>
	
	
</xsl:stylesheet>
