package it.uniroma1.dis.mfis.redia.verifint.control.binding;

import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control.provers.analysis.stateDiagram.SDNuSMVProver;
import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.ElementId;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDAction;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDAction_Factory;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDCondition;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDCondition_Factory;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDDiagram;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDElement;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDEvent;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDEvent_Factory;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDMacroState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDMacroState_Factory;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDState_Factory;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDTransition;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDTransition_Factory;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDDiagram_isComposedBy_SDElement;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDMacroState_containsInitState_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDMacroState_contains_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasAction_SDAction;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasCondition_SDCondition;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasDestination_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasEvent_SDEvent;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasOrigin_SDState;

import java.util.HashMap;
import java.util.List;
import java.util.ListIterator;
import java.util.SortedSet;

/**
 * @author Gianluca pc
 */
public class StateDiagramBinder implements Binder {
	
	private HashMap<String,String> hm;
	private SDDiagram diagram;
	private SDState_Factory state_factory;
	private SDAction_Factory action_factory;
	private SDEvent_Factory event_factory;
	private SDTransition_Factory transition_factory;
	private SDMacroState_Factory macrostate_factory;
	private SDCondition_Factory condition_factory;
	
	
	public StateDiagramBinder(String diagram_name){
		
		this.diagram = new SDDiagram(diagram_name);
		this.state_factory = new SDState_Factory();
		this.action_factory = new SDAction_Factory();
		this.event_factory = new SDEvent_Factory();
		this.transition_factory = new SDTransition_Factory();
		this.macrostate_factory = new SDMacroState_Factory();
		this.condition_factory = new SDCondition_Factory();
		hm = new HashMap<String,String>();
	}
	public HashMap<String,String> getHashMap(){
		return hm;
	}
	
	public void createState(String name){
		if(!hm.containsKey(name)){
			SDState state = this.state_factory.createInstance(this.diagram, name);
			hm.put(name,state.getIdentifierName());
		}
	}
	
	public SDState getInitState(){
		return diagram.getInitState();
	}
	
	public void setInitState(String name){
		if(hm.containsKey(name)){
			SDState state = (SDState)getElementByName(hm.get(name));
			if(state!=null){
				state.setInit();
			}
		}
	}
	
	/*TO DO
	public void createMacroState(String name, SDState initstate, List<SDState> states){
		if(!hm.containsKey(name)){
		SDMacroState macro = this.macrostate_factory.createInstance(diagram, initstate, name);
		ListIterator<SDState> it = states.listIterator();
		SDMacroState_containsInitState_SDState.addLink(macro, initstate);
		while(it.hasNext()){
			SDState state = it.next();
			SDMacroState_contains_SDState.addLink(macro, state);
		}
		hm.put(name,macro.getIdentifierName());
		}
	}
	*/
	
	public void createTransition(String origin, 
								 String dest, 
								 String event,
								 String action,
								 String condition){
		String key = null;
		if(condition!=null){
			key = origin+event+condition;
		}else{
			key = origin+event;
		}
		if(!hm.containsKey(key)){
		
		SDTransition t = transition_factory.createInstance(this.diagram, 
														   (SDState)getElementByName(diagram.getName()+"_state_"+origin),
														   (SDState)getElementByName(diagram.getName()+"_state_"+dest),
														   key);
		
		hm.put(key, t.getIdentifierName());
		
		if(!hm.containsKey(event)){
			SDEvent e = event_factory.createInstance(diagram, t, event);
			hm.put(event,e.getIdentifierName());
		}else{
			SDTransition_hasEvent_SDEvent.addLink(t, (SDEvent)getElementByName(diagram.getName()+"_event_"+event));
		}
		
		if(action!=null){
			if(!hm.containsKey(action)){
				SDAction a = action_factory.createInstance(diagram, t, action);
				hm.put(action,a.getIdentifierName());
			}else{
				SDTransition_hasAction_SDAction.addLink(t, (SDAction)getElementByName(diagram.getName()+"_action_"+action));
			}
			
		}
		
		if(condition!=null){
			if(!hm.containsKey(condition)){
				SDCondition c = condition_factory.createInstance(diagram, t, condition);
				hm.put(condition,c.getIdentifierName());
			}else{
				SDTransition_hasCondition_SDCondition.addLink(t, (SDCondition)getElementByName(diagram.getName()+"_condition_"+condition));
			}
		}
		}
	}
	
	public void deleteElement(String name){
		
    	SDElement element = (SDElement)getElementByName(hm.get(name));
		
    	if(element instanceof SDState){
    		deleteState((SDState)element);
    	}    	
    	
    	if(element instanceof SDTransition){
    		deleteTransition((SDTransition)element);
    	}
	}
	

	private void deleteState(SDState state){
		
		SDDiagram_isComposedBy_SDElement.removeLink(this.diagram, state);
		hm.remove(state.getName());
		
		SortedSet<SDTransition> transdest = state.getIsDestinations();
		for(SDTransition trans : transdest){
			deleteTransition(trans);
		}
		
		SortedSet<SDTransition> transorigin = state.getIsOrigins();
		for(SDTransition trans : transorigin){
			deleteTransition(trans);
		}
		
		//elminare i riferimenti nei macrostati! attenzione all' 1..1!
	}
	
	private void deleteTransition(SDTransition trans){
		
		SDDiagram_isComposedBy_SDElement.removeLink(this.diagram, trans);
		hm.remove(trans.getName());
		
		SDTransition_hasDestination_SDState.removeLink(trans, trans.getDestination());
		SDTransition_hasOrigin_SDState.removeLink(trans, trans.getOrigin());
		
		if(trans.getEvent().getTransitions().size()==1){ // se la transizione  l'unica per l'evento lo cancello
			SDDiagram_isComposedBy_SDElement.removeLink(this.diagram, trans.getEvent());
			hm.remove(trans.getEvent().getName());
		}
		SDTransition_hasEvent_SDEvent.removeLink(trans, trans.getEvent());
		
		hm.remove(trans.getName());
		
		if(trans.getAction()!=null){
			if(trans.getAction().getTransitions().size()==1){ // se la transizione  l'unica per l'azione la cancello	
				SDDiagram_isComposedBy_SDElement.removeLink(this.diagram, trans.getAction());
				hm.remove(trans.getAction().getName());
			}
			SDTransition_hasAction_SDAction.removeLink(trans, trans.getAction());
		}
		
		if(trans.getCondition()!=null){
			if(trans.getCondition().getTransitions().size()==1){ // se la transizione  l'unica per la condizione la cancello
				SDDiagram_isComposedBy_SDElement.removeLink(this.diagram, trans.getCondition());
				hm.remove(trans.getCondition().getName());
			}
			SDTransition_hasCondition_SDCondition.removeLink(trans, trans.getCondition());
		}

	}

    
    public Identifiable getElementByName(String name){
    	
    	SortedSet<Identifiable> elements = diagram.getElements();
    	SDElement result = null;
    	for (Identifiable identifiable : elements) {
    		SDElement element = (SDElement)identifiable;
    		if(element.getIdentifierName().equals(name)){
    			result = element;
    			break;
    		}
    	}
    	return result;	
    }

    public Identifiable getElementById(ElementId id){
    	
    	SortedSet<Identifiable> elements = diagram.getElements();
    	Identifiable result = null;
    	for (Identifiable identifiable : elements) {
    		if(identifiable.getId().equals(id)){
    			result = identifiable;
    			break;
    		}
    	}
    	return result;	
    }
   
    public ProvingResult prove(String testFormula){
    	return new SDNuSMVProver(this.diagram).prove(testFormula);
    }
    
}
