package it.uniroma1.dis.mfis.redia.verifint.control.provers.analysis.stateDiagram;

import it.uniroma1.dis.mfis.redia.verifint.control.ExecutionProperties;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProverProgramOutput;
import it.uniroma1.dis.mfis.redia.verifint.control._dataTypes.ProvingResult;
import it.uniroma1.dis.mfis.redia.verifint.control.io.XmlMarshaller;
import it.uniroma1.dis.mfis.redia.verifint.control.provers.Prover;
import it.uniroma1.dis.mfis.redia.verifint.control.provers._constants.NuSMVConstants;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams.SDDiagram;

import java.io.BufferedReader;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.InputStreamReader;
import java.io.PrintStream;

/**
 * @author Andrea Frasca
 */
public class SDNuSMVProver extends Prover {
	
	private String path_file;
	
	public SDNuSMVProver(SDDiagram diagram) {
		super(diagram);
		super.setExecutionProperties(
				NuSMVConstants.NUSMV_PROGRAM_PATH_KEY,
				NuSMVConstants.NUSMV_COMMAND_STRING_KEY,
				NuSMVConstants.NUSMV_XSLT_PATH_KEY);
		this.setOutputFile(
				ExecutionProperties.getInstance().getProperty(
						NuSMVConstants.NUSMV_FILE_SMV_PATH_KEY)
				);
	}
	
	private void setOutputFile(String path_file){
		this.path_file = path_file;
	}
	
	
	@Override
	public ProvingResult prove(String goal) {
		if (this.getCommandString().equals(EMPTY_COMMAND_STRING)
				||
				this.getProverProgramPath().equals(EMPTY_PROVER_PROGRAM_PATH)
				||
				this.getXsltPath().equals(EMPTY_XSLT_PATH)) {
			throw new IllegalStateException(
					EXECUTION_PROPERTIES_NOT_SET_EXCEPTION_MESSAGE);
		}
		
		String diagramXml =
			new XmlMarshaller().marshal((SDDiagram)this.diagram);
		
		String proverInput =
			createProverProgramInput(diagramXml);
		
		proverInput = proverInput.concat(goal);
		
		
		ProverProgramOutput proverProgramOutput =
			   proveByProgram(proverInput);
		
		String output = proverInput;
		output = output.concat(proverProgramOutput.output.concat(proverProgramOutput.errorOutput));
		
		ProvingResult tmpProvingResult =
			new ProvingResult(
					(proverProgramOutput.exitStatus == 0),
					output); 
		
		return tmpProvingResult;
		
	}
	
	@Override
	protected ProverProgramOutput proveByProgram(String proverInput){
		int exitStatus = 0;
		String outputText = "";
		String errorText = "";
		try {
			
			FileOutputStream file;
			PrintStream output;
			try {
				file = new FileOutputStream(path_file);
				output = new PrintStream(file);
				output.println(proverInput);
				output.close();
			}
			catch (IOException e) {
				System.err.println("Errore: " + e);
				System.exit(1);
			};
			
			// launch Prover
			Process process = Runtime.getRuntime().exec(
					buildProcessCommand(
							this.getCommandString() + " "+ path_file,
							this.getProverProgramPath()));
			/*
			 * Java 1.5 does not change the working directory, even though you 
			 * specified it; so, the following clean lines of code are useless,
			 * at least on Linux
			
			Process process = Runtime.getRuntime().exec(
					this.getCommandString(),
					new String[0],
					new File(this.getProverProgramPath()));
			 */
			/*
			PrintWriter inputWriter = new PrintWriter(
					process.getOutputStream());

			inputWriter.println(proverInput);
			inputWriter.flush();
			inputWriter.close();
			*/
			
			BufferedReader outputBufferedReader = new BufferedReader(
					new InputStreamReader(process.getInputStream()));
			String outputLine = outputBufferedReader.readLine();
			while (outputLine != null) {
				outputText += "\n" + outputLine;
				outputLine = outputBufferedReader.readLine();
			}
		
			BufferedReader errorBufferedReader = new BufferedReader(
					new InputStreamReader(process.getErrorStream()));
			String errorLine = errorBufferedReader.readLine();
			while (errorLine != null) {
				errorText += "\n" + errorLine;
				errorLine = errorBufferedReader.readLine();
			}			
			
		
			outputBufferedReader.close();
			errorBufferedReader.close();
			exitStatus = process.waitFor();
		}
		catch (Exception e) {
			System.err.println(e);
		}
		return new ProverProgramOutput(exitStatus, outputText, errorText);
		
	}
	
}
