package it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams;


import it.uniroma1.dis.mfis.redia.verifint.model.Diagram;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.XmlSerializable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDDiagram_isComposedBy_SDElement;

import java.io.Serializable;
import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlElements;
import javax.xml.bind.annotation.XmlRootElement;
import javax.xml.bind.annotation.XmlTransient;

/**
 * @author Andrea Frasca 
 */
@XmlRootElement
@XmlAccessorType(XmlAccessType.FIELD)
public class SDDiagram 
	implements
	Diagram,
	Comparable<SDDiagram>,
	Serializable,
	XmlSerializable{
	
	@XmlElement
	private String name;
	@XmlTransient
	private SortedSet<SDElement> elements;
	
	//@XmlTransient
	//private SortedSet<SDMacroState> macrostates;
	
	protected SDDiagram(){}
	
	public SDDiagram(String name) {
		this.name = name;
		this.elements = new TreeSet<SDElement>(); 
		//this.macrostates = new TreeSet<SDMacroState>();
	}
	
	public void addElement(
			SDDiagram_isComposedBy_SDElement
				cSDiagram_isComposedBy_SDElement) {
		this.elements.add(cSDiagram_isComposedBy_SDElement.getElement());
	}

	public void removeElement(
			SDDiagram_isComposedBy_SDElement
				cSDiagram_isComposedBy_SDElement) {
		this.elements.remove(
				cSDiagram_isComposedBy_SDElement.getElement());
	}
 
	protected void setElements(SortedSet<SDElement> elements) {
		this.elements = elements;
	}
    
	public int compareTo(SDDiagram o) {
		return this.name.compareTo(o.getName());
	}

	@Override
	public boolean equals(Object obj) {
		if (obj instanceof SDDiagram) {
			SDDiagram o = (SDDiagram) obj;
			return this.name.equals(o.getName());			
		}
		return false;
	}
	
	public SortedSet<Identifiable> getElements() {
		return new TreeSet<Identifiable>(this.elements);
	}
	/*
	public SortedSet<Identifiable> getMacroStates() {
		return new TreeSet<Identifiable>(this.macrostates);
	}*/
	
	public String getName() {
		return this.name;
	}

	public void setName(String name) {
		this.name = name;
	}
	
	@XmlElementWrapper(name="innerComponents")
	@XmlElements(value={
			@XmlElement(name="action", type=SDAction.class),
			@XmlElement(name="condition", type=SDCondition.class),
			@XmlElement(name="event", type=SDEvent.class),
			@XmlElement(name="activity", type=SDActivity.class)
	})
	public List<SDInnerComponent> getInnerComponents() {
		List<SDInnerComponent> innerComponents = new ArrayList<SDInnerComponent>();
		for (SDElement element : this.elements) {
			if (element instanceof SDInnerComponent) {
				innerComponents.add((SDInnerComponent)element);
			}
		}
		return innerComponents;
	}
	
	@XmlElementWrapper(name="components")
	@XmlElements(value={
			@XmlElement(name="state", type=SDState.class),
			@XmlElement(name="transition", type=SDTransition.class)
	})
	public List<SDComponent> getComponents() {
		List<SDComponent> components = new ArrayList<SDComponent>();
		for (SDElement element : this.elements) {
			if (element instanceof SDComponent && !(element instanceof SDMacroState)) {
				components.add((SDComponent)element);
			}
		}
		return components;
	}
	
	@XmlElementWrapper(name="macrostates")
	@XmlElements(value={
			@XmlElement(name="macrostate", type=SDMacroState.class),
	})
	public List<SDMacroState> getListMacroStates() {
		List<SDMacroState> macrostates = new ArrayList<SDMacroState>();
		for (SDElement element : this.elements) {
			if (element instanceof SDMacroState) {
				macrostates.add((SDMacroState)element);
			}
		}
		return macrostates;
	}
	
	public SDState getInitState() {
		SDState result = null;
		for (SDElement element : elements) {
			if(element instanceof SDState){
				if(((SDState)element).isInit()){
					result = (SDState)element;
					break;
				}
			}
		}
		return result;
	}
	
}
