package it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams;

import it.uniroma1.dis.mfis.redia.verifint.model._dataTypes.ElementId;
import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDDiagram_isComposedBy_SDElement;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._interfaces.XConstrainable;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._interfaces.XConstraint;
import it.uniroma1.dis.mfis.redia.verifint.model.constraints._links.XConstrainable_satisfies_XConstraint;

import java.io.Serializable;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.annotation.XmlAccessType;
import javax.xml.bind.annotation.XmlAccessorType;
import javax.xml.bind.annotation.XmlAttribute;
import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlID;
import javax.xml.bind.annotation.XmlTransient;

/**
 * @author Andrea Frasca
 */
@XmlAccessorType(XmlAccessType.FIELD)
public abstract class SDElement 
	implements Serializable, NameIdentifiable, XConstrainable {
	
		public static Long id_assigner = 1L;
		public static final String ELEMENT_PREFIX = "SD";
		public static final String ID_PREFIX = ELEMENT_PREFIX + "_Element_";
		public static final String DEFAULT_EMPTY_NAME = "";
		
		@XmlElement(required=false)
		private String name;
		@XmlAttribute(required=true)
		private ElementId id;
		@XmlTransient
		protected SDDiagram diagram;
		@XmlTransient
		private SortedSet<XConstraint> constraints;
		
		SDElement(String name) {
			synchronized (id_assigner) {
				this.setId(SDElement.id_assigner++);
			}
			this.name = name;
			this.constraints = new TreeSet<XConstraint>();
		}
		
		SDElement() {
			this(DEFAULT_EMPTY_NAME);
		}

		public String getName() {
			return this.name;
		}

		public void setName(String name) {
			this.name = name;
		}

		public ElementId getId() {
			return id;
		}

		private void setId(Long numericSuffix) {
			this.setId(new ElementId(ID_PREFIX + numericSuffix.toString()));
		}

		protected void setId(ElementId id) {
			this.id = id;
		}
		
		public SDDiagram getDiagram() {
			return this.diagram;
		}

		protected void setDiagram(SDDiagram diagram) {
			this.diagram = diagram;
		}
		
		public void setDiagram(SDDiagram_isComposedBy_SDElement link)
		throws AssociationTargetException {
			if (!link.getElement().equals(this))
				throw new AssociationTargetException(
						link.getClass().getName(),
						this.getClass().getName());
			this.diagram = link.getDiagram();
		}

		public void unsetDiagram(SDDiagram_isComposedBy_SDElement link)
		throws AssociationTargetException {
			if (!link.getElement().equals(this))
				throw new AssociationTargetException(
						link.getClass().getName(),
						this.getClass().getName());
			this.diagram = null;
		}
		
		public SortedSet<XConstraint> getConstraints() {
			return new TreeSet<XConstraint>(constraints);
		}

		protected void setConstraints(SortedSet<XConstraint> constraints) {
			this.constraints = constraints;
		}
		
		public void addConstraint(XConstrainable_satisfies_XConstraint link)
		throws AssociationTargetException {
			if (!link.getConstrainedElement().equals(this))
				throw new AssociationTargetException(
						link.getClass().getName(),
						this.getClass().getName());
			this.constraints.add(link.getConstraint());
		}

		public void removeConstraint(XConstrainable_satisfies_XConstraint link)
		throws AssociationTargetException {
			if (!link.getConstrainedElement().equals(this))
				throw new AssociationTargetException(
						link.getClass().getName(),
						this.getClass().getName());
			this.constraints.remove(link.getConstraint());
		}

		public int compareTo(Identifiable o) {
			return
				this.getId().compareTo(o.getId());
		}

		@Override
		public boolean equals(Object obj) {
			return
				this.getId().equals(((SDElement)obj).getId());
		}

		@Override
		public String toString() {
			return this.getIdentifierName();
		}
		
		@XmlAttribute
		@XmlID
		public String getIdentifierName() {
			return
				this.getDiagram().getName() +
				NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
				this.getName();
		}

}
