package it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams;

import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDMacroState_containsInitState_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDMacroState_contains_SDState;

import java.util.ArrayList;
import java.util.List;
import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.annotation.XmlElement;
import javax.xml.bind.annotation.XmlElementWrapper;
import javax.xml.bind.annotation.XmlElements;
import javax.xml.bind.annotation.XmlTransient;

/**
 * @author Andrea Frasca
 */
public class SDMacroState extends SDState {
	
	public static final String DEFAULT_IDENTIFIER_NAME_INFIX = "macrostate";
	
	@XmlTransient
	private SortedSet<SDState> states;

	private SDState initState;
	
	protected SDMacroState(){
		super();
	}
	
	protected SDMacroState(String name){
		super(name);
		states = new TreeSet<SDState>();
	}
	
	public SDState getInitialState(){
		return initState;
	}
	
	protected void setInitialState(SDState initiState){
		this.initState = initiState;
	}
	
	public void setInitState(SDMacroState_containsInitState_SDState link)
	throws AssociationTargetException {
		if (!link.getMacroState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.initState = link.getState();
		//devo imporre che esista anche nel soprainsieme contains
		SDMacroState_contains_SDState.addLink(link.getMacroState(), link.getState());
	}
	
	public void unsetInitState(SDMacroState_containsInitState_SDState link)
	throws AssociationTargetException {
		if (!link.getMacroState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.initState = null;
		//devo imporre che venga eliminato anche nel soprainsiseme contains
		SDMacroState_contains_SDState.removeLink(link.getMacroState(), link.getState());
	}
	
	public SortedSet<SDState> getStates() {
		return new TreeSet<SDState>(this.states);
	}

	protected void setStates(SortedSet<SDState> _states) {
		this.states = new TreeSet<SDState>(_states);
	}
	
	public void addState(SDMacroState_contains_SDState link)
	throws AssociationTargetException {
		if (!link.getMacroState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.states.add(link.getState());
	}

	public void removeState(SDMacroState_contains_SDState link)
	throws AssociationTargetException {
		if (!link.getMacroState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		//controllo che non sia contenuto in initState altrimenti non posso eliminarlo!
		if(!initState.equals(link.getState())){
			this.states.remove(link.getState());
		}
	}
	
	@XmlElementWrapper(name="states")
	@XmlElements(value={
			@XmlElement(name="state", type=SDState.class),
	})
	public List<SDState> getListStates() {
		List<SDState> list = new ArrayList<SDState>();
		for (SDState element : this.states) {
			list.add(element);
		}
		return list;
	}
	
	@Override
	public String getIdentifierName() {
		return
			this.getDiagram().getName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			DEFAULT_IDENTIFIER_NAME_INFIX+
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			this.getName();
	}
	
}
