package it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams;

import it.uniroma1.dis.mfis.redia.verifint.model._exceptions.AssociationTargetException;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.Identifiable;
import it.uniroma1.dis.mfis.redia.verifint.model._interfaces.NameIdentifiable;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDMacroState_containsInitState_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDMacroState_contains_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDState_hasActivity_SDActivity;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasDestination_SDState;
import it.uniroma1.dis.mfis.redia.verifint.model.analysis.stateDiagrams._link.SDTransition_hasOrigin_SDState;

import java.util.SortedSet;
import java.util.TreeSet;

import javax.xml.bind.annotation.XmlTransient;

/**
 * @author Andrea Frasca
 */
public class SDState extends SDComponent{
	
	public static final String DEFAULT_IDENTIFIER_NAME_INFIX = "state";
	
	private boolean isInit;
	private boolean isFinal;
	
	@XmlTransient
	private SortedSet<SDActivity> activities;
	@XmlTransient
	private SortedSet<SDTransition> isOrigins;
	@XmlTransient
	private SortedSet<SDTransition> isDestinations;
	@XmlTransient
	private SortedSet<SDMacroState> isContained;
	@XmlTransient
	private SortedSet<SDMacroState> isInitOf; // is a subset of "isContained"
	
	protected SDState() {
		super();
	}
	
	protected SDState(String name){
		super(name);
		this.isInit = false;
		this.isFinal = false;
		this.activities = new TreeSet<SDActivity>();
		this.isOrigins = new TreeSet<SDTransition>();
		this.isDestinations = new TreeSet<SDTransition>();
		this.isContained = new TreeSet<SDMacroState>();
		this.isInitOf = new TreeSet<SDMacroState>();
	}
	
	public void setInit(){
		SDState oldInit = diagram.getInitState();
		if(oldInit!=null){
			oldInit.unsetInit();
		}
		this.isInit = true;
	}
	
	public void unsetInit(){
		this.isInit = false;
	}
	
	public void setFinal(boolean value){
		this.isFinal = value;
	}
	
	public void unsetFinal(){
		this.isFinal = false;
	}
	
	public boolean isInit(){
		return this.isInit;
	}
	
	public boolean isFinal(){
		return this.isFinal;
	}
	
	public SortedSet<SDActivity> getActivities() {
		return new TreeSet<SDActivity>(this.activities);
	}

	protected void setActivities(SortedSet<SDActivity> _activities) {
		this.activities = new TreeSet<SDActivity>(_activities);
	}
	
	public SortedSet<SDTransition> getIsOrigins() {
		return new TreeSet<SDTransition>(this.isOrigins);
	}

	protected void setIsOrigins(SortedSet<SDTransition> _isOrigins) {
		this.isOrigins = new TreeSet<SDTransition>(_isOrigins);
	}
	
	public SortedSet<SDTransition> getIsDestinations() {
		return new TreeSet<SDTransition>(this.isDestinations);
	}

	protected void setIsDestinations(SortedSet<SDTransition> _isDestinations) {
		this.isDestinations = new TreeSet<SDTransition>(_isDestinations);
	}
	
	public SortedSet<SDMacroState> getIsContained() {
		return new TreeSet<SDMacroState>(this.isContained);
	}

	protected void setIsContained(SortedSet<SDMacroState> _isContained) {
		this.isContained = new TreeSet<SDMacroState>(_isContained);
	}
	
	public SortedSet<SDMacroState> getIsInitOf() {
		return new TreeSet<SDMacroState>(this.isInitOf);
	}

	protected void setIsInitOf(SortedSet<SDMacroState> _isInitOf) {
		this.isContained = new TreeSet<SDMacroState>(_isInitOf);
	}
	
	public void addActivity(SDState_hasActivity_SDActivity link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.activities.add(link.getActivity());
	}

	public void removeActivity(SDState_hasActivity_SDActivity link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.activities.remove(link.getActivity());
	}
	
	public void addIsOrigin(SDTransition_hasOrigin_SDState link)
		throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isOrigins.add(link.getTransition());
	}

	public void removeIsOrigin(SDTransition_hasOrigin_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isOrigins.remove(link.getTransition());
	}
	
	public void addIsDestination(SDTransition_hasDestination_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isDestinations.add(link.getTransition());
	}

	public void removeIsDestination(SDTransition_hasDestination_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isDestinations.remove(link.getTransition());
	}
	
	public void addMacroState(SDMacroState_contains_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isContained.add(link.getMacroState());
	}

	public void removeMacroState(SDMacroState_contains_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isContained.remove(link.getMacroState());
	}
	
	public void addMacroStateWhoIsInit(SDMacroState_containsInitState_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isInitOf.add(link.getMacroState());
		//bisogna aggiungerlo anche al soprainsieme
		SDMacroState_contains_SDState.addLink(link.getMacroState(), link.getState());
	}
	
	// non dovrebbe essere usato poich ho 1 come cardinalit minima
	public void removeMacroStateWhoIsInit(SDMacroState_containsInitState_SDState link)
	throws AssociationTargetException {
		if (!link.getState().equals(this))
			throw new AssociationTargetException(
					link.getClass().getName(),
					this.getClass().getName());
		this.isInitOf.remove(link.getMacroState());
		SDMacroState_contains_SDState.removeLink(link.getMacroState(), link.getState());
	}
	
	@Override
	public String getIdentifierName() {
		return
			this.getDiagram().getName() +
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			DEFAULT_IDENTIFIER_NAME_INFIX+
			NameIdentifiable.IDENTIFIER_NAME_SEPARATOR +
			this.getName();
	}

}
