/*
 * DecisioneFrame.java
 *
 * Created on 3 ottobre 2008, 18.11
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;

import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.awt.Polygon;
import java.util.ArrayList;
import java.util.List;
import java.util.LinkedList;
import java.awt.geom.Line2D;
/**
 *
 * @author Gianluca pc
 */
public class DecisionFrame extends Figure{
    private List<Figure> figures;
    private Color fillColor;
    private Color borderColor;
    private int xoffset = 5; // margine orizzontale rispetto al bordo
    private int interline = 5; // in pixel
    private Polygon poly;
    private LinkedList isTail;
    private LinkedList isHead;
    /**
     * Crea una nuova istanza di Cornice in una data posizione
     * @param position posizione grafica della cornice
     */
    public DecisionFrame(Point position)
    {        
        super(position);
        this.fillColor = new Color(248, 240, 186);
        this.borderColor = Color.BLACK;
        figures = new ArrayList<Figure>();
        Polygon poly;
        isTail = new LinkedList();
        isHead = new LinkedList();
    }
    
     public void addTailEdge(EdgeST e){
        isTail.add(e);
    }
    
    public void addHeadEdge(EdgeST e){
        isHead.add(e);
    }
    
    public Polygon getPolygon(){
        return poly;
    }
    
    @Override
    public void draw(Graphics2D g)
    {        
        dimension.height = interline; dimension.width = 0;        
        FontMetrics metrics;
        for(Figure f : figures)
        {                    
            if(f instanceof Label) {
                Label e = (Label) f;
                metrics = g.getFontMetrics(e.getFont());
                dimension.height+=metrics.getHeight() + interline;
                if(metrics.stringWidth(e.getText()) > 
                        dimension.width) 
                    dimension.width = metrics.stringWidth(
                            e.getText());
            } else {
                dimension.height+=interline * 2;
            }
        }        
        
        dimension.width+=(xoffset * 2); // Aggiungo un margine orizzontale      
        poly = new Polygon();
        poly.addPoint(position.x - 15,position.y);
        poly.addPoint(position.x + dimension.width/2,position.y  - 15 - dimension.height/2 );
        poly.addPoint(position.x + dimension.width + 15 ,position.y );
        poly.addPoint(position.x + dimension.width/2,position.y + 15 + dimension.height/2);
        poly.translate(0,11);
        g.setColor(new Color(210, 210, 210));
        g.fillPolygon(poly);
        if (selected) g.setColor(Color.MAGENTA);
        else g.setColor(getFillColor());
        g.fillPolygon(poly);
        g.setColor(getBorderColor());
        g.drawPolygon(poly);
        
        // Posizioni dell'etichetta calcolate in base all'allineamento
        int px = 0, py = position.y; 
        for(Figure f : figures)
        {   
            if(f instanceof Label) {
                Label e = (Label) f;
                py = py + g.getFontMetrics(e.getFont()).getHeight();
                switch(e.getTextAlignment())
                {
                    case Label.ALIGNMENT_CENTER:
                        px = (dimension.width / 2) - 
                                (g.getFontMetrics(
                                e.getFont()).stringWidth(e.getText()) / 2) 
                                + position.x;                    
                    break;
                    case Label.ALIGNMENT_RIGHT:
                        px = dimension.width - 
                                g.getFontMetrics(e.getFont()).stringWidth(
                                e.getText()) + position.x - xoffset;                    
                    break;
                    case Label.ALIGNMENT_LEFT:
                    default:
                        px = position.x + xoffset;
                    break;
                }            
                e.setPosition(new Point(px, py));
                e.draw(g);
            } else {
                py+=interline;
                g.setColor(this.borderColor);
                f.position.x = this.position.x;
                f.position.y = py;
                f.dimension.width = this.dimension.width;
                f.dimension.height = 0;
                f.draw(g);
            }
            py+=interline;
            
        }
    }
    
    @Override
    public Point hookPoint(Point p){
        Line2D l = new Line2D.Float(p, this.getCG());
        Point intersezione = getLineIntersection(l);
        if(intersezione == null) {
            return new Point(position.x - 15, position.y + 11);
        } else return intersezione;
    }
    
    /**
     * Calcola il punto di intersezione tra il bordo della figura e una linea
     * data
     * @param linea linea che si interseca con il bordo della figura
     * @return punto di intesezione tra la linea data e il bordo della figura
     */
    @Override
    public Point getLineIntersection(Line2D linea) {
        /* Attenzione: Si tratta di una funzione ad-hoc che trova soltanto la 
         * prima intersezione, indicato solo per rette che intersecano 
         * un rettangolo  solo una volta!
         * */
         Line2D line1;
         Line2D line2;
         Line2D line3;
         Line2D line4;
         Point intersection = new Point();
         
         int x = this.position.x;
         int y = this.position.y;
         int width = this.dimension.width;
         int height = this.dimension.height;
         Point Point1 = new Point(x - 15,y + 11);
         Point Point2 = new Point(x + width/2,y - 4 - height/2);
         Point Point3 = new Point(x + width + 15,y + 11);
         Point Point4 = new Point(x + width/2,y + 26 + height/2);
         line1 = new Line2D.Float(Point1,Point2);
         line2 = new Line2D.Float(Point2,Point3);
         line3 = new Line2D.Float(Point3,Point4);
         line4 = new Line2D.Float(Point4,Point1);
         intersection = lineIntersection(linea, line1);
         if(intersection != null) return intersection;
         intersection = lineIntersection(linea, line2);
         if(intersection != null) return intersection;
         intersection = lineIntersection(linea, line3);
         if(intersection != null) return intersection;
         intersection = lineIntersection(linea, line4);
         if(intersection != null) return intersection;
       
        return null;
    }
    
    
    
    
    /**
     * Restituisce la lista delle figure (etichette e separatori noti come Figure)
     * all'interno della cornice.
     * @return lista delle figure contenute nella cornice
     */
    public List<Figure> getFigure()
    {
        return figures;
    }

    /**
     * Imposta il colore di riempimento della cornice
     * @param fillColor colore di riempimento della cornice
     */
    public void setFillColor(Color fillColor) {
        this.fillColor = fillColor;
    }

    /**
     * Imposta il colore di bordo della cornice
     * @param borderColor colore di bordo della cornice
     */
    public void setBorderColor(Color borderColor) {
        this.borderColor = borderColor;
    }
    
    /**
     * Restituisce il colore di riempimento attuale della cornice
     * @return colore di riempimento attuale della cornice
     */
    public Color getFillColor() {
        return fillColor;
    }

    
    /**
     * Restituisce il colore di bordo attuale della cornice
     * @return colore di bordo attuale della cornice
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
 /*   public Polygon getPolygon() {
        return poly;
    }*/
    
    @Override
    public int compareTo(Object o)
    {
        if(this != o)
            return 1;
        else return 0;
    }
    
//    public void setPoisition(Point p,LinkedList toDraw){
//        super.setPosition(p);
//        for(int i = 0; i < isTail.size(); i++){
//            toDraw.remove(isTail.get(i));
//            ((EdgeST)isTail.get(i)).translateStartPoint();
//            toDraw.add(isTail.get(i));
//            
//        }
//        for(int j = 0; j < isHead.size(); j++){
//            toDraw.remove(isTail.get(j));
//            ((EdgeST)isTail.get(j)).translateEndPoint();
//            toDraw.add(isTail.get(j));
//        }
//    }
    
}

