/*
 * StateFrame.java
 *
 * Created on 4 dicembre 2008, 0.42
 *
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

package it.uniroma1.dis.mfis.redia.verifint.view.graphics.engine;
import java.awt.Color;
import java.awt.FontMetrics;
import java.awt.Graphics2D;
import java.awt.Point;
import java.util.ArrayList;
import java.util.List;
import java.awt.geom.Line2D;
import java.awt.Color;
import java.awt.Point;
import java.util.List;
import java.util.LinkedList;
import java.util.ListIterator;
import java.awt.geom.Rectangle2D;
/**
 *
 * @author Gianluca pc
 */
public class StateFrame extends Figure{
    private List<Figure> figures;
    private Color fillColor;
    private Color borderColor;
    private int xoffset = 5; // margine orizzontale rispetto al bordo
    private int interline = 5; // in pixel
    private Rectangle2D rect;
    private LinkedList<EdgeST> isTail;
    private LinkedList<EdgeST> isHead;
    private MacroStateFrame macroState;
    private boolean initial = false;
    
    /** Creates a new instance of StateFrame */
    public StateFrame(Point position) {
        super(position);
        this.fillColor = new Color(248, 240, 186);
        this.borderColor = Color.BLACK;
        figures = new ArrayList<Figure>();
        isTail = new LinkedList();
        isHead = new LinkedList();
    }
    
    public void addTailEdge(EdgeST e){
        isTail.add(e);
    }
    
    public void addHeadEdge(EdgeST e){
        isHead.add(e);
    }
    
    public Rectangle2D getRectangle(){
        return rect;
    }
    
    @Override
    public void draw(Graphics2D g)
    {        
        dimension.height = interline; dimension.width = 90;
        rect = new Rectangle2D.Double(position.x,position.x,dimension.width,dimension.height);
        
        if(isInitial()){
        	Color c = g.getColor();
        	g.setColor(Color.BLACK);
        	g.fillOval(position.x-30,position.y-30,10,10);
        	g.drawLine(position.x-28,position.y-28,position.x,position.y);
        	g.drawLine(position.x-15,position.y-5,position.x,position.y);
        	g.drawLine(position.x-5,position.y-15,position.x,position.y);
        	g.setColor(c);
        }
        
        FontMetrics metrics;
        for(Figure f : figures)
        {                    
            if(f instanceof Label) {
                Label e = (Label) f;
                metrics = g.getFontMetrics(e.getFont());
                dimension.height+=metrics.getHeight() + interline;
                if(metrics.stringWidth(e.getText()) > 
                        dimension.width) 
                    dimension.width = metrics.stringWidth(
                            e.getText());
            } else {
                dimension.height+=interline * 2;
            }
        }        
        
        dimension.width+=(xoffset * 2); // Aggiungo un margine orizzontale      
        g.setColor(new Color(210, 210, 210));
        g.fillRoundRect(position.x + 4, position.y + 4, dimension.width, 
                dimension.height,5,5);
        if (selected) g.setColor(Color.MAGENTA);
        else g.setColor(getFillColor());
        g.fillRoundRect(position.x, position.y, dimension.width, 
                dimension.height,5,5);
        g.setColor(getBorderColor());
        g.drawRoundRect(position.x, position.y, dimension.width, 
                dimension.height,5,5);
        
        // Posizioni dell'etichetta calcolate in base all'allineamento
        int px = 0, py = position.y; 
        for(Figure f : figures)
        {   
            if(f instanceof Label) {
                Label e = (Label) f;
                py = py + g.getFontMetrics(e.getFont()).getHeight();
                switch(e.getTextAlignment())
                {
                    case Label.ALIGNMENT_CENTER:
                        px = (dimension.width / 2) - 
                                (g.getFontMetrics(
                                e.getFont()).stringWidth(e.getText()) / 2) 
                                + position.x;                    
                    break;
                    case Label.ALIGNMENT_RIGHT:
                        px = dimension.width - 
                                g.getFontMetrics(e.getFont()).stringWidth(
                                e.getText()) + position.x - xoffset;                    
                    break;
                    case Label.ALIGNMENT_LEFT:
                    default:
                        px = position.x + xoffset;
                    break;
                }            
                e.setPosition(new Point(px, py));
                e.draw(g);
            } else {
                py+=interline;
                g.setColor(this.borderColor);
                f.position.x = this.position.x;
                f.position.y = py;
                f.dimension.width = this.dimension.width;
                f.dimension.height = 0;
                f.draw(g);
            }
            py+=interline;
            
        }
    }
    
    public List<Figure> getFigure()
    {
        return figures;
    }

    /**
     * Imposta il colore di riempimento della cornice
     * @param fillColor colore di riempimento della cornice
     */
    public void setFillColor(Color fillColor) {
        this.fillColor = fillColor;
    }

    /**
     * Imposta il colore di bordo della cornice
     * @param borderColor colore di bordo della cornice
     */
    public void setBorderColor(Color borderColor) {
        this.borderColor = borderColor;
    }
    
    /**
     * Restituisce il colore di riempimento attuale della cornice
     * @return colore di riempimento attuale della cornice
     */
    public Color getFillColor() {
        return fillColor;
    }

    
    /**
     * Restituisce il colore di bordo attuale della cornice
     * @return colore di bordo attuale della cornice
     */
    public Color getBorderColor() {
        return borderColor;
    }
    
 /*   public Polygon getPolygon() {
        return poly;
    }*/
    
    @Override
    public int compareTo(Object o)
    {
        if(this != o)
            return 1;
        else return 0;
    }
    
    public LinkedList getisTail(){
        return isTail;
    }
    
    public LinkedList getisHead(){
        return isHead;
    }
    
    public void setisTail(LinkedList tail){
        isTail = tail;
    }
    
    public void setisHead(LinkedList head){
        isHead = head;
    }
    
    
    public boolean remove(EdgeST e){
    	return isTail.remove(e) || isHead.remove(e);
    }
    
    public void setPosition(Point p,LinkedList toDraw){
        LinkedList provTail = new LinkedList();
        LinkedList provHead = new LinkedList();
        super.setPosition(p);
        for(int i = 0; i < isTail.size(); i++){
            toDraw.remove(isTail.element());
            if(isTail.element() instanceof EdgeLoop){
                EdgeLoop e = (EdgeLoop)isTail.element();
                EdgeLoop eNew = new EdgeLoop(this,e.getName());
                eNew.setStyle(Edge.STYLE_HAS);
                isTail.remove(e);
                isTail.add(eNew);
                toDraw.add(eNew);
            } else if(isTail.element() instanceof EdgeST){
                EdgeST e = (EdgeST)isTail.element();
                EdgeST eNew = new EdgeST(this,e.getHead(),new Line2D.Double(this.getCG(),e.getHead().getCG()),e.getName());
                eNew.setLabelName(e.getName());
                eNew.setStyle(Edge.STYLE_HAS);
                isTail.remove(e);
                ((StateFrame)e.getHead()).getisHead().remove(e);
                isTail.add(eNew);
                ((StateFrame)e.getHead()).getisHead().add(eNew);
                toDraw.add(eNew);
            }
            
            
        }
        for(int j = 0; j < isHead.size(); j++){
            toDraw.remove(isHead.element());
            EdgeST e = (EdgeST)isHead.element();
            EdgeST eNew = new EdgeST(e.getTail(),this,new Line2D.Double(e.getTail().getCG(),this.getCG()),e.getName());
            eNew.setLabelName(e.getName());
            eNew.setStyle(Edge.STYLE_HAS);
            isHead.remove(e);
            ((StateFrame)e.getTail()).getisTail().remove(e);
            isHead.add(eNew);
            ((StateFrame)e.getTail()).getisTail().add(eNew);
            toDraw.add(eNew);
        }
    }
    
    public void setInitial(){
    	initial = true;
    }
    
    public void unsetInitial(){
    	initial = false;
    }
    
    public boolean isInitial(){
    	return initial;
    }
  
    
}
